import { roundTo } from "../../../math";
import { CSSStyleValueTypes } from "./CSSStyle";
export function serializeValue(value, preferences) {
    var _a;
    switch (value.type) {
        case CSSStyleValueTypes.Color:
            return serializeValue(value.value, preferences);
        case CSSStyleValueTypes.Comment:
            return `/* ${value.text} */`;
        case CSSStyleValueTypes.FunctionCall:
            return (value.functionName + "(" + serializeValue(value.args, preferences) + ")");
        case CSSStyleValueTypes.Keyword:
            return value.ident;
        case CSSStyleValueTypes.List:
            return [value.head, ...value.tail]
                .map((v) => serializeValue(v, preferences))
                .join(value.separator);
        case CSSStyleValueTypes.Literal:
            return value.text;
        case CSSStyleValueTypes.Number:
            return (roundTo(value.value, (_a = value.precision) !== null && _a !== void 0 ? _a : preferences.decimalPlaces) +
                (value.unit || ""));
        case CSSStyleValueTypes.String:
            return `"${value.value.replace(/"/g, `\\"`)}"`;
        case CSSStyleValueTypes.Unknown:
            return value.text;
    }
}
export function serializeStyle(style, preferences) {
    return (style.propertyName + ": " + serializeValue(style.value, preferences) + ";");
}
